'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = generateBundles;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _hashModuleName = require('../lib/hashModuleName.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var distEntry = _path2.default.join(process.cwd(), 'src/dist.js');
var distOutDir = _path2.default.join(process.cwd(), 'dist');

var compiler = (0, _webpack2.default)({
  entry: [distEntry],
  output: {
    path: distOutDir,
    filename: _hashModuleName.hashedModuleName + '.js'
  },
  externals: {
    react: 'React',
    'react-dom': 'ReactDOM',
    aphrodite: 'aphrodite'
  },
  module: {
    loaders: [{
      test: /\.js?$/,
      loader: 'babel',
      query: {
        presets: ['react', 'es2015']
      }
    }]
  },
  plugins: [new _extractTextWebpackPlugin2.default(_hashModuleName.hashedModuleName + '.css'), new _webpack2.default.optimize.UglifyJsPlugin({
    compress: {
      warnings: false
    }
  })]
});

function template() {
  return '(function() {\n    var React = require(\'react\');\n    var ReactDOM = require(\'react-dom\');\n    var StyleSheet = require(\'aphrodite\').StyleSheet;\n    var component = require(\'./index.js\').default;\n    var factory = React.createFactory(component);\n    var mountNodes = document.querySelectorAll(\'.' + _hashModuleName.hashedClassName + '\');\n    [].forEach.call(mountNodes, (node) => {\n      let renderizrId = node.getAttribute(\'data-renderizr-id\');\n      StyleSheet.rehydrate(window.aphroditeClassNames[renderizrId]);\n      ReactDOM.render(factory(window.renderizrData[renderizrId]), node);\n    });\n  })();';
}

function createEntryFile() {
  return new Promise(function (res, rej) {
    _fs2.default.writeFile(distEntry, template(), { encoding: 'utf-8' }, function (err) {
      if (err) {
        return rej(err);
      }

      return res();
    });
  });
}

function compilerRun() {
  return new Promise(function (res, rej) {
    compiler.run(function (err, stats) {
      var jsonStats = stats.toJson();

      if (err || jsonStats.errors.length > 0) {
        return rej({
          fatal: err && err.stack,
          errors: jsonStats.errors.map(function (e) {
            return e.stack;
          })
        });
      }

      return res({
        js: _path2.default.join(distOutDir, _hashModuleName.hashedModuleName + '.js')
      });
    });
  });
}

function generateBundles() {
  return createEntryFile().then(compilerRun).catch(function (err) {
    return console.log(err, err.stack);
  });
}