'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = initialiser;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _crossSpawnAsync = require('cross-spawn-async');

var _crossSpawnAsync2 = _interopRequireDefault(_crossSpawnAsync);

var _pkgjson = require('./lib/pkgjson.js');

var _pkgjson2 = _interopRequireDefault(_pkgjson);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function copyTemplate(template, outDir) {
  return new Promise(function (res, rej) {
    _fsExtra2.default.copy(_path2.default.join(__dirname, '../templates/' + template), _path2.default.join(outDir, template), function (err) {
      if (err) return rej(err);
      return res();
    });
  });
}

function createRenderizrDirectory() {
  return new Promise(function (res, rej) {
    _fsExtra2.default.mkdirs(_path2.default.join(process.cwd(), '.renderizr'), function (err) {
      if (err) return rej(err);
      return res();
    });
  });
}

function createLambdaHandler() {
  return copyTemplate('lambda.js', process.cwd());
}

function createNoopRenderer() {
  return copyTemplate('renderer.js', _path2.default.join(process.cwd(), '.renderizr'));
}

function addDirToGitignore() {
  var pathToGitignore = _path2.default.join(process.cwd(), '.gitignore');
  return new Promise(function (res, rej) {
    _fsExtra2.default.createFile(pathToGitignore, function (err) {
      if (err) return rej(err);

      return _fsExtra2.default.readFile(pathToGitignore, function (readError, data) {
        if (readError) return rej(readError);
        if (data.toString().indexOf('.renderizr') !== -1) res();

        var newData = data;

        if (newData.length === 0) {
          newData = '.renderizr/\n';
        } else {
          newData = newData + '\n.renderizr';
        }

        return _fsExtra2.default.writeFile(pathToGitignore, newData, function (writeError) {
          if (writeError) return rej(writeError);
          return res();
        });
      });
    });
  });
}

function createLambda() {
  var AWS_REGION = process.env.AWS_REGION;


  return new Promise(function (res, rej) {
    var spawnOpts = {
      cwd: process.cwd(),
      stdio: [process.stdin, process.stdout, process.stderr]
    };
    var child = (0, _crossSpawnAsync2.default)(_path2.default.join(process.cwd(), 'node_modules/.bin/claudia'), ['create', '--name', _pkgjson2.default.name, '--region', AWS_REGION, '--api-module', 'lambda', '--version', 'DONOTUSE'], spawnOpts);

    child.on('error', function () {
      return rej();
    });
    child.on('exit', function (code) {
      if (code === 1) return rej();
      return res();
    });
  });
}

function log(message) {
  return function () {
    return console.log(message);
  };
}

function initialiser() {
  console.log('initialise all the things');
  return createRenderizrDirectory().then(log('Created .renderizr directory')).then(createLambdaHandler).then(log('Created lamda.js')).then(createNoopRenderer).then(log('Created .renderizr/renderer.js')).then(addDirToGitignore).then(log('Added .renderizr to .gitignore')).then(createLambda).then(log('Pushed lambda.js to AWS Lambda')).then(function () {
    return console.log('Renderized!');
  }).catch(function (err) {
    return console.log('I encountered the following error: ' + err.stack);
  });
}