'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = generateRenderer;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _generateEntryTemplate = require('./generate-entry-template.js');

var _generateRendererConfig = require('./generate-renderer-config.js');

var _hashModuleName = require('../lib/hashModuleName.js');

var _moduleMainFile = require('../lib/moduleMainFile.js');

var _generateRawRenderer = require('./generate-raw-renderer.js');

var _wrapComponent = require('./wrap-component.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* take the entry template, interpolate variables into it.
 * generate the raw entry file from the interpolated template
 * run the entry file through webpack to generate the renderer
 */

function createRenderer() {
  var entryPath = _path2.default.join(process.cwd(), '.renderizr/rawRenderer.js');
  var config = (0, _generateRendererConfig.generateRendererConfig)(entryPath);

  return new Promise(function (res, rej) {
    (0, _webpack2.default)(config, function (err, stats) {
      if (err) return rej(err.stack);
      var jsonStats = stats.toJson();
      if (jsonStats.errors.length > 0) return rej(jsonStats.errors);

      return res(_path2.default.join(process.cwd(), '.renderizr/renderer.js'));
    });
  });
}

function generateRenderer() {
  var entryPath = _path2.default.join(process.cwd(), '.renderizr/rawRenderer.js');
  var entryTemplate = (0, _generateEntryTemplate.generateEntryTemplate)(_moduleMainFile.mainFile, _hashModuleName.hashedClassName, _wrapComponent.wrapComponent);

  return (0, _generateRawRenderer.generateRawRenderer)(entryPath, entryTemplate).then(createRenderer).catch(function (err) {
    return console.log(err);
  });
}