'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.default = renderizeModule;

var _awsSdk = require('aws-sdk');

var _awsSdk2 = _interopRequireDefault(_awsSdk);

var _render = require('./render');

var _render2 = _interopRequireDefault(_render);

var _bundle = require('./bundle');

var _bundle2 = _interopRequireDefault(_bundle);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _crossSpawnAsync = require('cross-spawn-async');

var _crossSpawnAsync2 = _interopRequireDefault(_crossSpawnAsync);

var _pkgjson = require('./lib/pkgjson.js');

var _pkgjson2 = _interopRequireDefault(_pkgjson);

var _claudia = require('./lib/claudia.js');

var _claudia2 = _interopRequireDefault(_claudia);

var _mimeTypes = require('mime-types');

var _mimeTypes2 = _interopRequireDefault(_mimeTypes);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function putAssetInBucket(awsRegion, s3Bucket, asset) {
  return new Promise(function (res, rej) {
    _fs2.default.readFile(asset, function (readError, fileContent) {
      if (readError) return rej(readError);

      var s3 = new _awsSdk2.default.S3();
      var file = _path2.default.basename(asset);
      var type = _mimeTypes2.default.lookup(file);
      var params = {
        Bucket: s3Bucket,
        Key: file,
        Body: fileContent,
        ACL: 'public-read',
        ContentType: type
      };

      return s3.putObject(params, function (putError) {
        if (putError) return rej(putError);
        return res('https://s3-' + awsRegion + '.amazonaws.com/' + s3Bucket + '/' + file);
      });
    });
  });
}

function putAssetsInBuckets(js) {
  var awsRegion = process.env.AWS_REGION;
  var s3Bucket = process.env.RENDERIZR_BUNDLES_S3_BUCKET;
  var uploadAsset = putAssetInBucket.bind(putAssetInBucket, awsRegion, s3Bucket);

  return Promise.all([uploadAsset(js)]);
}

function createLambda() {
  var awsRegion = process.env.AWS_REGION;
  var lambdaVersion = _pkgjson2.default.version.replace(/\./g, '_');
  var lambdaId = _claudia2.default.api.id;

  return new Promise(function (res, rej) {
    var spawnOpts = {
      cwd: process.cwd(),
      stdio: [process.stdin, process.stdout, process.stderr]
    };
    var child = (0, _crossSpawnAsync2.default)(_path2.default.join(process.cwd(), 'node_modules/.bin/claudia'), ['update', '--version', lambdaVersion], spawnOpts);
    child.on('error', function () {
      return rej();
    });
    child.on('exit', function (code) {
      if (code === 1) return rej();
      return res('https://' + lambdaId + '.execute-api.' + awsRegion + '.amazonaws.com/' + lambdaVersion + '/render');
    });
  });
}

function publishSnsMessage(renderer, js) {
  return new Promise(function (res, rej) {
    var awsRegion = process.env.AWS_REGION;
    var sns = new _awsSdk2.default.SNS({
      region: awsRegion
    });
    var snsTopicOwner = process.env.SNS_OWNER;
    var registrationsTopic = process.env.SNS_TOPIC;

    var params = {
      Message: JSON.stringify({
        name: _pkgjson2.default.name,
        version: _pkgjson2.default.version,
        renderer: renderer,
        js: js,
        css: 'foo'
      }),
      TopicArn: 'arn:aws:sns:' + awsRegion + ':' + snsTopicOwner + ':' + registrationsTopic
    };

    sns.publish(params, function (err, data) {
      if (err) return rej(err);
      return res(data);
    });
  });
}

function renderizeModule() {
  return Promise.all([(0, _render2.default)().then(function () {
    return createLambda();
  }), (0, _bundle2.default)().then(function (_ref) {
    var js = _ref.js;
    return putAssetsInBuckets(js);
  })]).then(function (_ref2) {
    var _ref3 = _slicedToArray(_ref2, 2);

    var renderer = _ref3[0];

    var _ref3$ = _slicedToArray(_ref3[1], 1);

    var js = _ref3$[0];
    return publishSnsMessage(renderer, js);
  }).catch(function (err) {
    return console.log('Error: ', err, err.stack);
  });
}